import { GeoApiClient } from './GeoApiClient.ts'
import { beforeEach, describe, it } from '@std/testing/bdd'
import { assertEquals, assertRejects } from '@std/assert'
import { restore, returnsNext, stub } from '@std/testing/mock'
import { aPlaceDetailsReponse } from './tests/fixtures.ts'

const invalidResponse = {}

export type ResponsePayload = Record<string, unknown> | Array<unknown> | Error

export const fakeFetch = (response: ResponsePayload) => {
  return stub(
    globalThis,
    'fetch',
    returnsNext([Promise.resolve(new Response(JSON.stringify(response)))]),
  )
}

describe('GeoAPIClient', () => {
  let client: GeoApiClient

  beforeEach(() => {
    restore()
    client = new GeoApiClient('http://holi.geo.gedoenz.doesntexist', console)
  })

  it('throws error for invalid response when fetching geometry', async () => {
    using _ = fakeFetch(invalidResponse)

    await assertRejects(() => client.resolveCoordinates('somePlaceId'))
  })

  it('fetches lat lon for place id', async () => {
    using _ = fakeFetch(aPlaceDetailsReponse)

    const response = await client.resolveCoordinates('somePlaceId')

    assertEquals(response, {
      lat: 53.550341,
      lon: 10.000654,
    })
  })

  it('fetches city name of place id', async () => {
    using _ = fakeFetch(aPlaceDetailsReponse)

    const response = await client.resolveCityName('somePlaceId')

    assertEquals(response, 'Hamburg')
  })
})
